
-- geometry related functions

local function isLineObstructed(xstart, ystart, xend, yend)
    local vx, vy = makeVector(xstart, ystart, xend, yend)
    local sx, sy = vector_setLength(vx, vy, TILE_SIZE) -- normalize to tile size
    local steps = vector_getLength(vx, vy) / TILE_SIZE
    
    local x, y = xstart, ystart
    for i = 0, steps do
        if isObstructed(x, y) then
            spawnParticleEffect("awesomedebug2", x, y)
            return true, x, y
        end
        spawnParticleEffect("awesomedebug", x, y)
        x = x + sx
        y = y + sy
    end
    return false, x, y
end

local toPolarAngle = math.atan2

-- center* : center point (map coords)
-- d* : direction vector from circle center (which counts as zero-angle) -- relative!
-- angle:  max. allowed angles between [-up ... +down]
-- p* : target point (map coords)
local function isPointInArc(centerx, centery, dx, dy, angleUp, angleDown, px, py)

    -- if in right quadrant, angles must be swapped to deliver correct result
    if dx > 0 then
        angleUp, angleDown = angleDown, angleUp
    end
    
    local prx, pry = makeVector(centerx, centery, px, py) -- point relative vector from center
    prx, pry = vector_normalize(prx, pry)
    dx, dy = vector_normalize(dx, dy)
    local ppol = toPolarAngle(prx, pry)
    local dpol = toPolarAngle(dx, dy)
    
    local aup = DEGTORAD * angleUp
    local adown = DEGTORAD * angleDown
    
    local a = ppol - dpol
    --if a >= -aup and a <= adown then spawnParticleEffect("awesomedebug2", px, py) else spawnParticleEffect("awesomedebug", px, py) end
    return a >= -aup and a <= adown
    
end

return {
    isLineObstructed = isLineObstructed,
    isPointInArc = isPointInArc,
}
