
local Elena = v.createChar()
Elena.name = "Elena"
Elena.flag = FLAG_CHAR_ELENA
Elena.entityname = "elena"
Elena.id = CHAR_ELENA


local SHOTTIMER = 1.2
local WEAK_TIME = 120

Elena.shots = 0
Elena.justshot = false
Elena.light = 0
Elena.spirittimer = 0



Elena.init = function(entity)
    Elena.updatefuncs = {}

    
    Elena.me = entity
    
    -- IDEA: dynamic shot amount based on upgrades?
    Elena.shots = {}
    for i = 1,3 do
        Elena.shots[i] = { timer = 0 }
    end
    
    loadSound("mutantnaija-note")
    loadSound("mutantnaija-note-hit")
end


Elena.postInit = function()
    Elena.me = v.n
end

Elena.start = function()
    entity_alpha(v.n, 1)
    entity_alpha(Elena.entity, 0)
    
    setCanChangeForm(true)
    setCostume("mutant") -- also reverts to normal form
    setCanChangeForm(false)
    
    avatar_setBlockSinging(true)
    
    Elena.light = createQuad("softglow-add", 13)
    local s1 = 1.9
    local s2 = 2.2
    quad_scale(Elena.light, s1, s1)
    quad_scale(Elena.light, s2, s2, 1.2, -1, 1)
    quad_setBlendType(Elena.light, BLEND_ADD)

    if Elena.spirittimer > 0 then
        entity_alpha(v.n, 0.4, 0.7, -1, 1)
    end
end

Elena.save = function()

end

Elena.stop = function()
    quad_delete(Elena.light, 0.5)
    
    entity_alpha(Elena.entity, 1)

    --v.storeCurrentHealth(Elena)

    --entity_alpha(v.n, 0)
    --setFlag(FLAG_NAIJA_FORM, getForm())

    --change form?
    
    --avatar_setBlockSinging(true)
    
    -- TODO: set current char
    --entity_setPosition(v.n, entity_getPosition(Chars.curEnt))
end


local function shoot()
    local mx, my = getMouseWorldPos()
    local x = mx - entity_x(Elena.me)
    local y = my - entity_y(Elena.me)
    local targ = v.getNearestEntityToCursor(nil, 300, ET_ENEMY, nil, Elena.me)
    if targ ~= 0 and v.isDebugVis() then
        entity_doGlint(targ, "Particles/PurpleFlare")
    end
    local s = createShot("elena", Elena.me, targ)
    shot_setAimVector(s, x, y)
end

local function shootCharged()
    local mx, my = getMouseWorldPos()
    local ex, ey = entity_getPosition(Elena.me)
    local x = mx - ex
    local y = my - ey
    x, y = vector_normalize(x, y)
    local targ = v.getNearestEntityToCursor(nil, 300, ET_ENEMY, nil, Elena.me)
    if targ ~= 0 and v.isDebugVis() then
        entity_doGlint(targ, "Particles/PurpleFlare")
    end
    local s = createShot("elena", Elena.me, targ)
    shot_setAimVector(s, x, y)
    
    local a = (math.atan2(x, y) * RADTODEG) + 90
    local spread = 0.1
    
    local nx, ny = v.vector_rotateRad(x, y, spread)
    s = createShot("elena-nosound", Elena.me, targ)
    shot_setAimVector(s, nx, ny)
    
    nx, ny = v.vector_rotateRad(x, y, -spread)
    s = createShot("elena-nosound", Elena.me, targ)
    shot_setAimVector(s, nx, ny)
    
    if avatar_isBursting() then
        nx, ny = v.vector_rotateRad(x, y, -spread * 2)
        s = createShot("elena-nosound", Elena.me, targ)
        shot_setAimVector(s, nx, ny)
        
        nx, ny = v.vector_rotateRad(x, y, spread * 2)
        s = createShot("elena-nosound", Elena.me, targ)
        shot_setAimVector(s, nx, ny)
    end
    
    playVisualEffect(VFX_RIPPLE, ex, ey)
    spawnParticleEffect("greenpower", ex, ey) -- FIXME: this could look better
end

local function chargeVisual()
    local ex, ey = entity_getPosition(Elena.me)
    spawnParticleEffect("chomperlunge", ex, ey)
end

Elena.rmbtimer = 0
Elena.chargevis = false
Elena.justrmb = false
Elena.charging = false

Elena.update = function(dt)
    for _,f in pairs(Elena.updatefuncs) do
        f(dt)
    end
    
    if Elena.light ~= 0 then
        quad_setPosition(Elena.light, getMouseWorldPos())
    end
    
    local rmb = isRightMouse()
    
    local justclicked = rmb and not Elena.holdrmb and getNodeToActivate() == 0 -- do not shoot when saving, for example then
    local justreleased = not rmb and Elena.holdrmb and getNodeToActivate() == 0
    
    if rmb then
        Elena.rmbtimer = Elena.rmbtimer + dt

        if not Elena.charging and Elena.rmbtimer > 0.25 then
            Elena.charging = true
            v.avatar_startEmitter("elenacharging")
        end
        
        if Elena.rmbtimer >= SHOTTIMER and not Elena.chargevis then
            chargeVisual()
            Elena.chargevis = true

            entity_playSfx(Elena.me, "powerup")
            v.avatar_stopEmitter("elenacharging")
            v.avatar_startEmitter("elenacharging2")
        end
    else

        if Elena.charging then
            Elena.charging = false
            v.avatar_stopEmitter("elenacharging")
            v.avatar_stopEmitter("elenacharging2")
        end
    
        if Elena.rmbtimer >= SHOTTIMER then
            for _,s in pairs(Elena.shots) do
                s.timer = SHOTTIMER
            end
            shootCharged()
        end
        Elena.chargevis = false
        Elena.justshot = false
        Elena.rmbtimer = 0
    end
    
    
    for _,s in pairs(Elena.shots) do
        if s.timer >= dt then
            s.timer = s.timer - dt
        else
            if justreleased and not Elena.justshot then
                s.timer = SHOTTIMER
                shoot()
                Elena.justshot = true
                break
            end
        end
    end
    
    Elena.holdrmb = rmb
    
    -- input handling end ---
    
    -- TODO: not sure if this should be kept - poison/blind immunity
    -- if Naija and Elena are in the same group, and both selectable to play,
    -- this can be abused by switching back and forth.
    cureAllStatus()
    
    
    -- death handler
    if entity_getHealth(v.n) <= 0 then
        if Elena.spirittimer <= 0 then
            debugLog("Elena death - switching to spirit")
            
            Elena.spirittimer = WEAK_TIME -- HMM: save this as variable to prevent cheating?
            shakeCamera(18, 1)
            
            -- HACK: prevent death from actually kicking in
            entity_revive(v.n, 0.25)
            
            watch(0.2)
            entity_idle(v.n) -- HACK WTF
            entity_changeHealth(v.n, 0.2 - entity_getHealth(v.n))
            screenFadeGo(1.5)
            
            createEntity("logichelp_elenaspirit")
            -- reverted by entity
            setCanChangeForm(true)
            learnSong(SONG_SPIRITFORM)
            changeForm(FORM_SPIRIT)
            debugLog("Elena death - spirit transition done")
            
            entity_alpha(v.n, 0.4, 0.7, -1, 1)
        else
            Elena.spirittimer = WEAK_TIME -- prevent the timer from expiring while real death scene should kick in

            -- does not seem to work
            --if entity_getAnimationName(v.n) ~= "diepainfully" then
            --    entity_animate(v.n, "diepainfully")
            --end
        end
    end
    
    if Elena.spirittimer > 0 then
        Elena.spirittimer = Elena.spirittimer - dt
	if Elena.spirittimer <= 0 then
	    entity_alpha(v.n, 1, 1)
        end
    end

end


v.chars[CHAR_ELENA] = Elena
