
local Naija = v.createChar()

Naija.name = "Naija"
Naija.flag = FLAG_CHAR_NAIJA
Naija.entityname = "naija2"
Naija.id = CHAR_NAIJA

Naija.oldhealth = 999 
Naija.needCostumeFix = false -- Naija.start() is also called when changing maps, and it would always revert back to normal form. this bool delays that until we are in normal form.

Naija.chargeEffT = 0

Naija.lhandq = 0
Naija.rhandq = 0
Naija.energyshockq = 0

Naija.leftarm = 0
Naija.rightarm = 0

Naija.wasrmb = 0
Naija.chargeT = 0

local function learnSongIfStored(song)
    local flag = Naija.songtab[song]
    if getFlag(flag) ~= 0 then
        learnSong(song)
        setFlag(flag, 0)
    end
end

local function unlearnSongAndStore(song)
    local flag = Naija.songtab[song]
    if hasSong(song) then
        unlearnSong(song)
        setFlag(flag, 1)
    else
        setFlag(flag, 0)
    end
end

local function toggleBuiltinSongs(on)

    -- custom songs can be ignored, as they can be used only with the singing interface,
    -- and that is blocked if not playing as Naija.

    local isStored = getFlag(FLAG_SONGS_STORED)
    if on ~= (isStored ~= 0) then
        debugLog("toggleBuiltinSongs early return, isStored: " .. isStored)
        return false
    end
    
    if on then
        for s,_ in pairs(Naija.songtab) do
            learnSongIfStored(s)
        end
        setFlag(FLAG_SONGS_STORED, 0) -- just turned on, saved values are now invalid
    else
        for s,_ in pairs(Naija.songtab) do
            unlearnSongAndStore(s)
        end
        setFlag(FLAG_SONGS_STORED, 1) -- just turned off, so we saved the songs
    end
end

local function healthHappiness()
    local newhealth = entity_getHealth(Naija.me)
    if newhealth > Naija.oldhealth + 0.9 then
        if Naija.oldhealth <= 2 then
            v.pushTQ(0.7, function() emote(EMOTE_NAIJASIGH) end) -- uffffffff
        else
            -- better to leave this off, its a bit too much
            --v.pushTQ(0.7, function() emote(EMOTE_NAIJAGIGGLE) end) -- yay
        end
    end
    Naija.oldhealth = newhealth
end

local function saveCostume()
    local costume = getCostume()
    if costume == "mutant" then
        -- HACK: have seen this happen in *rare* cases
        -- simply prevent saving it should do
        debugLog("Naija saveCostume HACK - prevented " .. costume)
        return
    end
    debugLog("Naija save costume: " .. costume)
    setStringFlag("Naija_costume", costume)
end

local function restoreCostume()
    local costume = getStringFlag("Naija_costume")
    debugLog("Naija: restore costume " .. costume)
    setCostume(costume)
end

local function createHandGlows()
    if Naija.lhandq == 0 then
        Naija.energyshockq = createQuad("softglow-add", 13)
        Naija.lhandq = createQuad("softglow-add", 13)
        Naija.rhandq = createQuad("softglow-add", 13)
        quad_setBlendType(Naija.lhandq, BLEND_ADD)
        quad_setBlendType(Naija.rhandq, BLEND_ADD)
        quad_setBlendType(Naija.energyshockq, BLEND_ADD)
        quad_color(Naija.lhandq,  0.95, 0.45, 0.08)
        quad_color(Naija.rhandq,  0.95, 0.45, 0.08)
        quad_color(Naija.energyshockq,  0.95, 0.45, 0.08)
        local s = 0.5
        quad_scale(Naija.lhandq, s, s)
        quad_scale(Naija.rhandq, s, s)
        quad_alpha(Naija.lhandq, 0)
        quad_alpha(Naija.rhandq, 0)
        quad_alphaMod(Naija.lhandq, 0.6)
        quad_alphaMod(Naija.rhandq, 0.6)
        quad_alpha(Naija.energyshockq, 0)
    end
end

local function deleteHandGlows()
    if Naija.lhandq ~= 0 then
        quad_delete(Naija.lhandq)
        quad_delete(Naija.rhandq)
        quad_delete(Naija.energyshockq)
        Naija.lhandq = 0
        Naija.rhandq = 0
        Naija.energyshockq = 0
    end
end

local function updateExtras(dt)

    local rmb = isRightMouse()
    if rmb then
        Naija.chargeT = avatar_getSpellCharge()
    end
    
    if isForm(FORM_FISH) or isForm(FORM_SPIRIT) then
        deleteHandGlows()
    else
        createHandGlows()
    end

    if Naija.lhandq ~= 0 then
        if isForm(FORM_ENERGY) then
            quad_alpha(Naija.lhandq, 1, 1)
            quad_alpha(Naija.rhandq, 1, 1)
            
            if not rmb and Naija.wasrmb and Naija.chargeT > 1.5 then
                Naija.chargeT = 0
                local qs = Naija.energyshockq
                --local qs = createQuad("softglow-add", 13)
                --quad_setBlendType(qs, BLEND_ADD)
                --quad_color(qs,  0.95, 0.45, 0.08)
                quad_alpha(qs, 1)
                quad_alpha(qs, 0, 0.6)
                quad_scale(qs, 2, 2)
                quad_scale(qs, 5, 5, 0.4)
                --quad_delete(qs, 0.4)
            end
        else
            quad_alpha(Naija.lhandq, 0, 0.3)
            quad_alpha(Naija.rhandq, 0, 0.3)
        end
        
        -- FIXME: remove this later (see bug below)
        Naija.leftarm = entity_getBoneByIdx(Naija.me, 11)
        Naija.rightarm = entity_getBoneByIdx(Naija.me, 10)
    
        local x, y = entity_getPosition(Naija.me) -- FIXME use .entity
        local lx, ly = bone_getWorldPosition(Naija.leftarm)
        local rx, ry = bone_getWorldPosition(Naija.rightarm)
        quad_setPosition(Naija.lhandq, lx, ly)
        quad_setPosition(Naija.rhandq, rx, ry)
        quad_setPosition(Naija.energyshockq, x, y)
    end
    
    Naija.wasrmb = rmb
end

-----------------------------------------------------------------------

Naija.init = function()
    Naija.updatefuncs = { healthHappiness, updateExtras }
    
    Naija.songtab = {} -- idx | flag
    Naija.songtab[SONG_SHIELD] = FLAG_SONG_SHIELD
    Naija.songtab[SONG_BIND] = FLAG_SONG_BIND
    Naija.songtab[SONG_ENERGYFORM] = FLAG_SONG_ENERGY
    Naija.songtab[SONG_BEASTFORM] = FLAG_SONG_BEAST
    Naija.songtab[SONG_NATUREFORM] = FLAG_SONG_NATURE
    Naija.songtab[SONG_SUNFORM] = FLAG_SONG_SUN
    Naija.songtab[SONG_FISHFORM] = FLAG_SONG_FISH
    Naija.songtab[SONG_SPIRITFORM] = FLAG_SONG_SPIRIT
    --Naija.songtab[SONG_DUALFORM] = FLAG_SONG_DUAL
    
    -- activate hand glow
    createHandGlows()
end


Naija.postInit = function()
    Naija.me = v.n
    
    -- FIXME: Naija.entity is not set right here
    Naija.leftarm = entity_getBoneByName(Naija.entity, "leftarm")
    Naija.rightarm = entity_getBoneByName(Naija.entity, "rightarm")
end

Naija.start = function()
    entity_alpha(v.n, 1)
    entity_alpha(Naija.entity, 0)
    
    avatar_setBlockSinging(false)
    setCanChangeForm(true)
    
    if isForm(FORM_NORMAL) then
        restoreCostume()
    else
        Naija.needCostumeFix = true
        debugLog("Naija -- delay costume restore")
    end
    
    --changeForm(getFlag(FLAG_NAIJA_FORM))
    toggleBuiltinSongs(true)
    
    Naija.oldhealth = entity_getHealth(v.n)
end

Naija.save = function()
    toggleBuiltinSongs(false)
    saveCostume()
end

Naija.stop = function()
    if Naija.entity ~= 0 then
        entity_alpha(Naija.entity, 1)
    end
end


Naija.update = function(dt)
    for _,f in pairs(Naija.updatefuncs) do
        f(dt)
    end
    
    if Naija.needCostumeFix and isForm(FORM_NORMAL) then
        Naija.needCostumeFix = false
        restoreCostume()
    end
    
    updateExtras(dt)
    
end


v.chars[CHAR_NAIJA] = Naija
