-- Copyright (C) 2007, 2010 - Bit-Blot
--
-- This file is part of Aquaria.
--
-- Aquaria is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
-- See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-- FG: slightly modified, ehanced athmosphere
-- also added a little gimmick related to naija energyform charge

if not v then v = {} end
if not AQUARIA_VERSION then dofile("scripts/entities/entityinclude.lua") end

dofile("scripts/inc_util.lua")
dofile("scripts/inc_flagfunc.lua")

v.lights = 0
v.ntimer = 0

local function createLight(me, s1, s2, amod, layer)
    local light
    
    --local r = entity_getRotation(me)
    --local x, y = v.vector_fromDeg(r, 50)
    --debugLog(string.format("LAMP rot: %.3f, x: %.3f, y: %.3f", r, x, y))
    ----- rotation is always 0.000 WTF ?
    
    local light = createQuad("glow-add", layer)
    quad_setBlendType(light, BLEND_ADD)
    quad_scale(light, s1, s1)
    quad_scale(light, s2, s2, 2, -1, 1)
    quad_color(light, 0.95, 0.45, 0.08)
    quad_setPosition(light, entity_x(me), entity_y(me))
    quad_alphaMod(light, amod)
    table.insert(v.lights, light)
    return light
end

local function addLights(me, layer)
    createLight(me, 2.8, 3.3, 0.1, layer)
    createLight(me, 5, 6, 0.1, layer)
    createLight(me, 7, 9, 0.1, layer)
end

local function handleNaijaCharge(me)
    if (isForm(FORM_ENERGY) or isForm(FORM_SUN)) and avatar_getSpellCharge() > 0.5 and entity_touchAvatarDamage(me, 256) then
        local p = v.calcNaijaPower()
        if p > 0.5 then
            entity_damage(me, me, 999)
            spawnParticleEffect("energylampzap", entity_x(me), entity_y(me))
        end
    end
end

function init(me)
	setupBasicEntity(
	me,
	"breakable/energylamp",		-- texture
	1,							-- health
	1,							-- manaballamount
	1,							-- exp
	1,							-- money
	90,							-- collideRadius
	STATE_IDLE,					-- initState
	128,						-- sprite width	
	256,						-- sprite height
	1,							-- particle "explosion" type, maps to particleEffects.txt -1 = none
	0,							-- 0/1 hit other entities off/on (uses collideRadius)
	2000						-- updateCull -1: disabled, default: 4000
	)

	loadSound("energylamp-explode")
	entity_setDeathScene(me, true)
	entity_setDeathSound(me, "energylamp-explode")

	entity_setEatType(me, EAT_NONE)
	
	entity_setState(me, STATE_IDLE)
	
	entity_setUpdateCull(me, 3000)
	esetv(me, EV_LOOKAT, 0)
	
	entity_setDamageTarget(me, DT_AVATAR_PET, false)
	entity_setDamageTarget(me, DT_AVATAR_LIZAP, false)
    
    v.lights = {}
    addLights(me)
    addLights(me, 13)
end

function postInit(me)
end

function update(me, dt)
	entity_handleShotCollisions(me)
    if v.ntimer < dt then
        handleNaijaCharge(me)
        v.ntimer = 0.5
    else
        v.ntimer = v.ntimer - dt
    end
end

function enterState(me)
	if entity_isState(me, STATE_IDLE) then
	elseif entity_isState(me, STATE_DEATHSCENE) then
		spawnParticleEffect("starexplode", entity_x(me)+10, entity_y(me)+50)
		for i=1,3 do
			debugLog("Spawning")
			local e = createEntity("BrokenPiece", "", entity_x(me), entity_y(me))
			local str = string.format("%s-0001", "breakable/energylamp", i)
			--debugLog(str)
			entity_setTexture(e, str)
		end
		entity_setStateTime(me, 0.1)
        
        if v.lights ~= 0 then
            for _, light in pairs(v.lights) do
                 quad_delete(light , 1)
            end
            v.lights = 0
        end
	end
end

function exitState(me)
end

function damage(me, attacker, bone, damageType, dmg)
	return true
end

function msg(me, s, x)
    if s == "bg" then
        entity_switchLayer(me, -4)
        entity_setAllDamageTargets(me, false)
    end
end
        

function animationKey(me, key)
end

function hitSurface(me)
end

function songNote(me, note)
end

function songNoteDone(me, note)
end

function song(me, song)
end

function activate(me)
end

