-- Copyright (C) 2007, 2010 - Bit-Blot
--
-- This file is part of Aquaria.
--
-- Aquaria is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
-- See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-- FG: hacked

if not v then v = {} end
if not AQUARIA_VERSION then dofile("scripts/entities/entityinclude.lua") end

dofile("scripts/inc_flags.lua")

local NOTE_TIME = 2.5

local MAX_NOTES = 8
local MIN_DIST = 356

v.myNote = 0
v.timingNote = false
v.noteTimer = 0
v.core = 0
v.shell1 = 0
v.shell2 = 0
v.glow = 0
v.glowquad = 0

v.clickable = false

local STATE_SHAKE = 1000

local function glowNormal(me)
	bone_alpha(v.glow, 0.3)
	bone_alpha(v.glow, 0.4, 1, -1, 1, 1)
	bone_scale(v.glow, 6, 6)
	bone_scale(v.glow, 8, 8, 1, -1, 1, 1)
end

local function glowSinging(me)
	bone_alpha(v.glow, 0.5)
	bone_alpha(v.glow, 0.7, 0.2, -1, 1, 1)

	bone_scale(v.glow, 6, 6)
	bone_scale(v.glow, 24, 24, 0.2, -1, 1, 1)
end

-- this is not even a hash function lol
local function calcMyNote(me)
    local x, y = entity_getPosition(me)
    x = math.abs(x * 73)
    y = math.abs(y * 5.7)
    local z = math.floor(x + y)
    local note = z % MAX_NOTES
    debugLog("healthupgrade: hash = " .. z .. " -- note = " .. note)
    return note
end

function init(me)
	-- set color based on note
	setupEntity(me, "", "")
	entity_setEntityType(me, ET_NEUTRAL)
	entity_initSkeletal(me, "HealthUpgrade")
	
	
	v.core = entity_getBoneByName(me, "Core")
	v.shell1 = entity_getBoneByName(me, "Shell1")
	v.shell2 = entity_getBoneByName(me, "Shell2")
	v.glow = entity_getBoneByName(me, "Glow")
	
    v.myNote = calcMyNote(me)

	
	bone_setColor(v.core, getNoteColor(v.myNote))
	bone_setColor(v.shell1, getNoteColor(v.myNote))
	bone_setColor(v.shell2, getNoteColor(v.myNote))
	
	bone_setBlendType(v.glow, BLEND_ADD)
	bone_setColor(v.glow, getNoteColor(v.myNote))

	glowNormal(me)
	
	
	entity_scale(me, 0.6, 0.6)
	
	entity_setState(me, STATE_IDLE)
	
	entity_setEntityLayer(me, -1)
    
    --entity_setActivation(me, AT_NONE, 128) -- FIXME: this fucks up for some reason

    entity_setAllDamageTargets(me, false)
    
    local noglow = entity_getNearestNode(me, "noglow")
    if not (noglow ~= 0 and node_isEntityIn(noglow, me)) then
        v.glowquad = createQuad("glow-add", 13)
        quad_setPosition(v.glowquad, entity_getPosition(me))
        quad_setBlendType(v.glowquad, BLEND_ADD)
        quad_color(v.glowquad, getNoteColor(v.myNote))
        quad_scale(v.glowquad, 2.5, 2.5)
        quad_scale(v.glowquad, 3,3 , 2, -1, 1)
    end
end

function postInit(me)
	if entity_isFlag(me, 1) then
        v.entDebugVis(me)
        if isFlag(FLAG_DBG_SPAWNALWAYS, 0) then
            entity_delete(me)
        end
	end
end

v.incut = false

local function collect(me)
    v.incut = true
    playSfx("HealthUpgrade-Collect")
    spawnParticleEffect("HealthUpgradeReceived", entity_getPosition(me))
    
    if entity_isFlag(me, 0) then
        local hp = tonumber(v.getCharValue("maxhp"))
        debugLog("healthupgrade: current: " .. hp)
        hp = hp + 0.5
        v.setCharValue("maxhp", hp)
        entity_setHealth(getNaija(), hp)
    end
    
    setSceneColor(1, 1, 1, 4)
    entity_idle(getNaija())
    entity_setFlag(me, 1)
    watch(3)
    
    if v.glowquad ~= 0 then
        quad_delete(v.glowquad, 2)
    end
    entity_delete(me, 1)
end

function update(me, dt)
	if v.incut then return end
	
	if entity_isState(me, STATE_OPENED) then
		if entity_isEntityInRange(me, getNaija(), 64) and entity_getHealth(getNaija()) < 10 then
            if v.isNaija() or v.isLucien() or v.isLi() or v.isElena() then -- more? in case there are others, they are only temporary and not supposed to collect it.
                collect(me)
            end
		end
	elseif entity_isState(me, STATE_OPEN) and not entity_isAnimating(me) then
		entity_setState(me, STATE_OPENED)
	elseif v.isNaija() or v.isLucien() then -- those can sing
        if v.clickable then
            entity_setActivationType(me, AT_NONE)
            v.clickable = false
        end
        
		if v.timingNote then
			v.noteTimer = v.noteTimer + dt
			if v.noteTimer > NOTE_TIME then
				v.noteTimer = 0
				v.timingNote = false
				entity_setState(me, STATE_OPEN)
			end
		end
	elseif not v.clickable and (v.isLi() or v.isElena()) then  -- the rest can't sing
        entity_setActivationType(me, AT_CLICK)
        v.clickable = true
    end
end

function activate(me)
    entity_setState(me, STATE_OPEN)
    entity_setActivationType(me, AT_NONE) -- desyncs but who cares
    --glowSinging(me) -- nah
end

function songNote(me, note)
    if entity_getDistanceToEntity(me, getNaija()) > MIN_DIST then return end
	if entity_getAlpha(me) < 1 then return end
	if entity_isState(me, STATE_OPEN) or entity_isState(me, STATE_OPENED) then return end
	if note == v.myNote then
		entity_setState(me, STATE_SHAKE)
		v.timingNote = true
		v.noteTimer = 0
	else
		v.timingNote = false
	end
	v.timer = 0
end

function songNoteDone(me, note, len)
	if entity_isState(me, STATE_OPEN) or entity_isState(me, STATE_OPENED) then return end
	if v.timingNote and note == v.myNote then		
		if not entity_isState(me, STATE_OPEN) then
			entity_setState(me, STATE_IDLE)
		end
	end
end

function enterState(me, state)
	--debugLog("HU enterState!")
	if entity_isState(me, STATE_IDLE) then
		if v.shell1 ~= 0 and v.shell2 ~= 0 then
			bone_alpha(v.shell1, 0)
			bone_alpha(v.shell2, 0)
		end
		v.timingNote = false
		v.noteTimer = 0
		entity_animate(me, "idle", LOOP_INF)
		
		glowNormal(me)
	elseif entity_isState(me, STATE_SHAKE) then
		glowSinging(me)
		entity_animate(me, "shake", LOOP_INF)
	elseif entity_isState(me, STATE_OPEN) then
		bone_alpha(v.core, 0.01, 0.5)
		bone_alpha(v.shell1, 1, 0.1)
		bone_alpha(v.shell2, 1, 0.1)
		entity_animate(me, "open")
		
		local r, g, b = getNoteColor(v.myNote)
		setSceneColor(r, g, b, 2)
		
		playSfx(getNoteName(v.myNote, "low-"))
	end
end

function exitState(me, state)
end
