-- Copyright (C) 2007, 2010 - Bit-Blot
--
-- This file is part of Aquaria.
--
-- Aquaria is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
-- See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-- FG: TODO: ignore player if in submarine

-- FIXME: they still rotate really weird, and swim into walls, and stuff. fix this. looks not good.
-- FIXME: must make naija visible again if the shark dies while eaten

if not v then v = {} end
if not AQUARIA_VERSION then dofile("scripts/entities/entityinclude.lua") end

dofile("scripts/inc_flags.lua")
dofile("scripts/inc_timerqueue.lua")

local STATE_CHASE = 1001

local NORMAL_SPEED = 300
local CHASE_SPEED = 500

local ROT_ADJUST_TIME = 0.4

v.dir = 1
v.jaw = 0

v.radjt = 0

v.redelay = 0

v.vx = 0
v.vy = 0

v.eaten = false

v.obst = 0
v.oldx = 0
v.oldy = 0

v.ignorenode = 0
v.ignt = 0 -- ignore node re-search timer

-- FIXME: make this more accurate
-- FIXME: wait, is this even used?
local function getMouthPos(me)
    --return entity_x(me) + v.jawOffsX, entity_y(me) + v.jawOffsY
    return bone_getWorldPosition(v.jaw)
end
    

function init(me)
    
    if chance(50) then
        v.dir = -1
    end
    
    v.biteDist = math.random(350, 460) -- evil: add a little random variance so it is unpredictable when the shark will consider biting

	setupBasicEntity(
	me,
	"",							-- texture
	100,							-- health
	2,							-- manaballamount
	2,							-- exp
	10,							-- money
	200,							-- collideRadius (for hitting entities + spells)
	STATE_IDLE,						-- initState
	128,							-- sprite width	
	128,							-- sprite height
	1,							-- particle "explosion" type, 0 = none
	0,							-- 0/1 hit other entities off/on (uses collideRadius)
	4000							-- updateCull -1: disabled, default: 4000
	)
	
	entity_initSkeletal(me, "Shark")	
	entity_generateCollisionMask(me)
	

	entity_setDeathParticleEffect(me, "Explode")
	
	entity_setState(me, STATE_IDLE)
	entity_setUpdateCull(me, -1)
	
	v.jaw = entity_getBoneByName(me, "Jaw")
    --v.body = entity_getBoneByIdx(me, 0)
    
    local s = math.random(1.85, 2.15)
    entity_scale(me, s, s)
    
    entity_setMaxSpeed(me, NORMAL_SPEED)
    
    -- save CPU if not on screen (makes quite a difference as a segmented texture is used)
    entity_setCull(me, true)

end

function postInit(me)
    v.n = getNaija()
    entity_setTarget(v.n)
end

local function isIgnore(me)
    return v.ignorenode ~= 0 and node_isEntityIn(v.ignorenode, v.n)
end

local function doChase(me, dt)

    if isIgnore(me) and not entity_isState(me, STATE_ATTACK) then
        if not entity_isState(me, STATE_IDLE) then
            entity_setState(me, STATE_IDLE)
        end
        return
    end

    if entity_isState(me, STATE_IDLE) or entity_isState(me, STATE_CHASE) then
		local inRange = false
        local inSight = false
		local x, y = bone_getWorldPosition(v.jaw)
        
        if entity_isPositionInRange(v.n, x, y, 1300) then
			inSight = true
            if entity_isPositionInRange(v.n, x, y, v.biteDist) then
                inRange = true
                debugLog("in jaw range")
            end
        end
        
		if v.dir < 0 then
			if entity_x(v.n) > entity_x(me) then
				inRange = false
                if inSight then
                    v.dir = -v.dir
                end
			end
		else
			if entity_x(v.n) < entity_x(me) then
				inRange = false
                if inSight then
                    v.dir = -v.dir
                end
			end
		end
        
        if inRange then
            entity_setState(me, STATE_ATTACK)
		elseif inSight and not entity_isState(me, STATE_CHASE) then
            entity_setState(me, STATE_CHASE)
        end
	end
    
end

local function handleEaten(me)
    -- must not change v.n visibility here, because other characters may be in play, too!
    -- use these instead!
    
    v.eaten = true
    fade(0,  0,  1,   0, 0)
    fade(1,  10, 0.33, 0, 0)
    spawnParticleEffect("tinyredexplode", entity_getPosition(v.n))
    emote(EMOTE_NAIJAUGH)
    playSfx("squishy-die")
    
    local e = v.getCharEntity()
    entity_alpha(e, 0)
    
    --TODO FIXME: block ring menu !!

    if v.isDebug() then
        v.pushTQ(3, function()
            playSfx("secret")
            v.eaten = false
            v.redelay = 3
            fade(0, 0.3)
            entity_alpha(e, 1)
        end)
    end
            
end

local function checkEaten(me)
    
    if v.isDebug() then
        return
    end
    
    local b = entity_getNearestBoneToPosition(me, entity_getPosition(v.n))
    if b ~= v.jaw or v.eaten then return end
    
    local bx, by = bone_getWorldPosition(b)
    local dx = entity_x(v.n) - bx
    local dy = entity_y(v.n) - by
    
    local dist = math.sqrt(dx*dx + dy*dy)
    
    if dist < 130 then
        handleEaten(me)
        return true
    end
    
    debugLog("checkEaten dist = " .. dist)
end

local function isStuck(me)
    --return entity_isNearObstruction(me, 1, OBSCHECK_RANGE) -- seems not to work
    local x, y = entity_getPosition(me)
    return math.abs(v.oldx - x) < 0.001 and math.abs(v.oldy - y) < 0.001
end

function update(me, dt)

    v.updateTQ(dt)
    
	if entity_isState(me, STATE_ATTACK) and not entity_isAnimating(me) then
		entity_setState(me, STATE_IDLE)
	end
    
    local isChase = false
    
    if v.redelay <= 0 and not v.eaten and not v.isDebug() then
        local e = v.getCharEntity() -- TODO: optimize this (use timer or so - do it less often)
        if entity_getAlpha(e) > 0.1 then -- ignore invis (or already eaten)
            doChase(me, dt)
            isChase = entity_isState(me, STATE_CHASE) or entity_isState(me, STATE_ATTACK)
        else
            v.redelay = 0.5
        end
    else
        v.redelay = v.redelay - dt
    end
    
    if not entity_isInterpolating(me) then
        if isChase then
            local x, y = vector_normalize(entity_getVectorToEntity(me, v.n))
            v.vx = x * 700
            v.vy = y * 700
            entity_addVel(me, v.vx, v.vy)
        elseif v.redelay > 0 then
            entity_addVel(me, v.vx, v.vy) -- re-use last speed
        else
            entity_addVel(me, 700*v.dir, 0)
        end
        
        entity_updateCurrents(me, dt) -- TODO: really?
        entity_updateMovement(me, dt)
    end

    
    local offs = 0 -- rotation correction - to make sure mouth is always going into naijas direction (looks weird without this)
    if isChase or entity_isState(me, STATE_ATTACK) then
        offs = 10
    end
    

	if not entity_isVelIn(me, 60) then
		entity_flipToVel(me)
        
        if entity_isfh(me) then
            entity_rotateToVel(me, 0, 270 - offs)
        else
            entity_rotateToVel(me, 0, 90 + offs)
        end
    end
	
	if isObstructed(entity_x(me) + 300*v.dir, entity_y(me)) then
		v.dir = -v.dir
	end
	
	entity_handleShotCollisionsSkeletal(me)
	local bone = entity_collideSkeletalVsCircle(me, v.n)
	if bone ~= 0 then
		if avatar_isTouchHit() then
            checkEaten(me)
            entity_damage(v.n, me, 1)
		end
	end
    
    if v.eaten then
        entity_setPosition(v.n, bone_getWorldPosition(v.jaw))
        entity_damage(v.n, me, 1) -- HACK: sometimes the game forgets to account collision damage when stuck in shark
    end
    
    if isChase then
        if isStuck(me) then
            v.obst = v.obst + dt
            if v.obst > 0.8 then
                debugLog("shark stuck, using wayfinding")
                v.obst = 0
                entity_swimToPosition(me, entity_x(v.n), entity_y(v.n))
            end
        else
            v.obst = 0
        end
    end
    
    v.oldx, v.oldy  = entity_getPosition(me)
    
    if v.ignt < dt then
        v.ignt = math.random(500, 1000) / 100 -- add variance to prevent all scanning at once, possibly causing lag
        v.ignorenode = entity_getNearestNode(me, "sharkignore")
    else
        v.ignt = v.ignt - dt
    end
end

function hitSurface(me)
	v.dir = -v.dir
end

function enterState(me)
	if entity_isState(me, STATE_IDLE) then
		entity_animate(me, "idle", LOOP_INF)
        entity_setMaxSpeed(me, NORMAL_SPEED)
        --debugLog("idle now")
	elseif entity_isState(me, STATE_ATTACK) then
		entity_animate(me, "bite")
        v.redelay = 2
        
        -- HMM: supernatural senses with slowmo? maybe make an ability for this.
        -- HMM: for now, i think its ok if only elena has this.
        if v.isElena() then
            setGameSpeed(0.3)
            setGameSpeed(1, 2.3)
            playSfx("naijagasp")
        end
        --debugLog("attack now")
    elseif entity_isState(me, STATE_CHASE) then
        entity_setMaxSpeed(me, CHASE_SPEED)
        entity_setMaxSpeedLerp(me, 1.66, 20)
        entity_animate(me, "chase", LOOP_INF)
        --debugLog("chase now")
    end
end

function animationKey(me, key)
	if entity_isState(me, STATE_ATTACK) then
		if key == 1 then
			entity_setMaxSpeedLerp(me, 7)
            entity_setMaxSpeedLerp(me, 1, 0.6)
			playSfx("bite", 0.2)
		end
	end
end

function exitState(me)
end

function dieNormal(me)
	if chance(100) then
		spawnIngredient("SharkFin", entity_x(me), entity_y(me))
	end
end

function damage(me, attacker)
	if entity_x(v.n) > entity_x(me) then
		v.dir = 1
	else
		v.dir = -1
	end
    if attacker == v.n then
        entity_moveTowardsTarget(me, 1, 1000)
        entity_setState(me, STATE_CHASE)
    end
	return true
end

function shiftWorlds(me, old, new)
    -- must restore alpha to make the spirit visible
    -- then, either revert back invis, or make opaque if escaped
    local e = v.getCharEntity()
    
    if old == WT_SPIRIT then
        if not entity_isEntityInRange(me, v.n, 220) then
            v.eaten = false
            entity_alpha(e, 1)
            fade(0, 0)
        end
        
        if v.eaten then
            entity_alpha(e, 0)
        end
    else
        entity_alpha(e, 1)
        --fade(0, 0)
    end
end
