
--- overall mod config part
--dofile("scripts/_loader.lua")




---- globally used states ( >= 800, < 1000 )----
STATE_SLEEP = 800

----- playable character IDs [FLAG_CURRENT_CHAR] ----
CHAR_NAIJA = 0
CHAR_LI = 1
CHAR_ELENA = 2
CHAR_LUCIEN = 3
CHAR_KROTITE = 4

---- character state IDs ----
CHARSTATE_ELSEWHERE = 0  -- not in group, not selectable. may come back later. this is the default state when starting the mod
CHARSTATE_PLAYABLE = 1   -- in group and selectable. logic_chars.lua will spawn an entity for this if on the same map as currently played char.
CHARSTATE_FORCE = 2      -- lock to this char. group may exist, but can't be switched. can only be set for 1 char. overrides PLAYABLE and ELSEWHERE. [TODO FIXME]
CHARSTATE_NONEXIST = 3   -- dead or whatever; not in group, not selectable, not playable.

---- crystal effect IDs ----
CE_NONE = 0
CE_POWER = 1
CE_CURE = 2
CE_ECHO = 3


---- values for setStory() ---- [TODO: FIXME]
STORY_BEGIN = 0                  -- not yet used.
STORY_NAIJA_MIAJAIL = 1          -- set by node "trans dreamtojail" on map startdream
STORY_LI_SEARCHING_NAIJA = 2     -- set by cutscenes on miajail, nxtitle, or the jailveil trap save point


---- main flags starting here. defining whether various events happened, etc ----

MOD_FLAGS_BEGIN = 301
-- #########################
FLAG_NAIJA_IN_CRYSTAL = 301 -- 0: no, 1: yes
-- .....

FLAG_BLOWUP_VOLCANO = 417
FLAG_CAN_USE_CRYSTAL_POWER = 418
FLAG_GREENCRYSTALS_DESTROYED = 419
-- #########################
MOD_FLAGS_END = 419


---- control flags, for node/entity communication ----
FLAG_MOD_INITED = 420
FLAG_CAN_SWITCH_FORM = 421 --- uhm...? FIXME
FLAG_NO_HEAT_DAMAGE = 422 -- checked in node_hotarea.lua
FLAG_CURRENT_CHAR = 423 -- CHAR_* value
FLAG_NAIJA_FORM = 424 -- form of naija before character change (OBSOLETE - can only change char in normal form)

FLAG_CHAR_NAIJA = 425 -- one of CHARSTATE_* -- this must be in the same order as CHAR_* constants!!
FLAG_CHAR_LI = 426
FLAG_CHAR_ELENA = 427
FLAG_CHAR_LUCIEN = 428
FLAG_CHAR_KROTITE = 429
FLAG_CHAR_START   = FLAG_CHAR_NAIJA
FLAG_CHAR_END     = FLAG_CHAR_KROTITE


FLAG_RELOAD_UI = 440 -- checked in logic_ui.lua
FLAG_BREAK_SLEEP = 441 -- used to enforce waking up - checked in node_sleep.lua. 0: default; 1: break sleep; 2: force sleep
FLAG_ACTIVE_CRYSTAL_POWER = 442 -- read + reset by logichelp_entitydeath.lua
--FLAG_CHARCHANGE_MAPTRANS = 443 -- used by logic_chars.lua. if ~= 0 the current map change was inited by a character change.
FLAG_ENABLE_RING_UI = 444 -- controls whether char selection & debug ring menu is available
FLAG_BLOCK_CHANGE_CHAR = 445 -- if set, logic_chars.lua will never change current char. Reset on every init.

---- stats and scripted secondary upgrades ----
FLAG_HAS_ENERGY2 = 460
FLAG_HAS_SUNBEAM = 461
FLAG_HAS_SHIELD2 = 462
FLAG_LI_FLASHLIGHT = 463 -- NYI

---- songs backup store for character change ----
-- TODO: move this to a "Naija_songs" string flag array
FLAG_SONG_SHIELD = 480
FLAG_SONG_BIND = 481
FLAG_SONG_ENERGY = 482
FLAG_SONG_BEAST = 483
FLAG_SONG_NATURE = 484
FLAG_SONG_SUN = 485
FLAG_SONG_FISH = 486
FLAG_SONG_SPIRIT = 487
FLAG_SONG_DUAL = 488 -- dual form NOT USED! -- TODO: replace with something more fitting
FLAG_SONGS_STORED = 489 -- safeguard against accidental overwriting on character switch


---- debug flags ----
FLAG_DBG_SPAWNALWAYS = 490 -- spawn story-related things even if they were already destroyed
FLAG_DBG_VISUALS = 491 -- display visual indicators for various things
FLAG_DBG = 492 -- allow cheating for debugging (opening doors and stuff). also prevents cutscenes from changing the map, etc etc.
FLAG_DBG_TOGGLE = 493 -- 0: ignore, 1: enable debug, 2: disable debug (auto-resets to 0)




-- ### misc values ### --

MOD_MAX_GREENCRYSTALS = 8 -- FIXME: this must be set to the actual amount in game
-- HMM: if crystals can be planted and their position is recorded; this must be adjusted accordingly.
-- HMM: not sure if it should be possible to create more crystals, but it would certainly add a little something

function v.setNumFlag(flag, val)
    setStringFlag(flag, tostring(val))
end

function v.getNumFlag(flag)
    local val = tonumber(getStringFlag(flag))
    if not val then
        val = 0
    end
    return val
end

function v.isModFlag(x) -- OBSOLETE?
    return x >= MOD_FLAGS_BEGIN and x <= MOD_FLAGS_END
end

function v.isDebug()
    return getFlag(FLAG_DBG) ~= 0
end

function v.isDebugVis()
    return getFlag(FLAG_DBG_VISUALS) ~= 0
end

v._dbgVis = 0

function v.debugVis(x, y, tex, scale)
    if v._dbgVis ~= 0 then
        quad_delete(v._dbgVis)
        v._dbgVis = 0
    end
    
    if not v.isDebugVis() then
        return
    end
    
    if tex == true then
        tex = "yes"
    elseif tex == false then
        tex = "no"
    elseif not tex or tex == "" then
        tex = "missingimage"
    end
    local q = createQuad(tex)
    if scale then
        quad_scale(q, scale, scale)
    end
    
    quad_setPosition(q, x, y)
    v._dbgVis = q 
end

function v.nodeDebugVis(me, tex, scale)
    local x, y = node_getPosition(me)
    v.debugVis(x, y, tex, scale)
end

function v.entDebugVis(me, tex, scale)
    local x, y = entity_getPosition(me)
    v.debugVis(x, y, tex, scale)
end

function v.edbg(me, s, line)
    if v.isDebugVis() then
        local nl = ""
        if line then
            while line ~= 0 do
                nl = nl .. "\n"
                line = line - 1
            end
        end
        s = nl .. s
        entity_debugText(me, s)
        entity_debugText(me, s)
        -- twice, otherwise its too transparent
    end
end

function v.setCharacter(c, delayed)
    if delayed then c = -c end
    setFlag(FLAG_CURRENT_CHAR, c) -- negative numbers indicate the char should be switched on map change.
end

function v.getCharacter()
    return getFlag(FLAG_CURRENT_CHAR)
end

function v.getCharName()
    return getStringFlag("Char_name") -- set by logic_chars.lua
end

function v.getCharValue(var)
    local t = v.getCharName() .. "_" .. var
    local val = getStringFlag(t)
    debugLog("getCharValue[" .. t .. "] = " .. val)
    return val
end

function v.setCharValue(var, val)
    local t = v.getCharName() .. "_" .. var
    local s = tostring(val)
    debugLog("setCharValue[" .. t .. "] = " .. s)
    return setStringFlag(t, s)
end

function v.isNaija()
    return getFlag(FLAG_CURRENT_CHAR) == CHAR_NAIJA
end

function v.isLucien()
    return getFlag(FLAG_CURRENT_CHAR) == CHAR_LUCIEN
end

function v.isElena()
    return getFlag(FLAG_CURRENT_CHAR) == CHAR_ELENA
end

function v.isLi()
    return getFlag(FLAG_CURRENT_CHAR) == CHAR_LI
end

function v.getCharEntity()
    if v.isNaija() or v.isElena() then
        return v.n
    end
    
    local name = getStringFlag("Char_entname") -- set by logic_chars.lua
    local e = getEntity(name)
    if e == 0 then
        centerText("ERROR: v.getCharEntity() for name: " .. name)
    end
    return e
end
    
function v.charId_isPlayable(c)
    if c < FLAG_CHAR_START then -- auto-correct if CHAR_* instead of FLAG_CHAR_* was passed
        debugLog("charId_isPlayable: auto-adjusting char id " .. c .. " to flag") -- but warn about it
        c = c + FLAG_CHAR_START
    end
    
    if c >= FLAG_CHAR_START and c <= FLAG_CHAR_END then
        local fv = getFlag(c)
        debugLog("charId_isPlayable: flag " .. c .. " is " .. fv)
        return fv == CHARSTATE_PLAYABLE or fv == CHARSTATE_FORCE
    else
        centerText("charId_isPlayable(" .. c .. ") fail")
        return false
    end
end

function v.forcePlayCharacter(c)
    if c < FLAG_CHAR_START then -- auto-correct if CHAR_* instead of FLAG_CHAR_* was passed
        debugLog("forcePlayCharacter: auto-adjusting char id " .. c .. " to flag") -- but warn about it
        c = c + FLAG_CHAR_START
    end
    
    for i = FLAG_CHAR_START, FLAG_CHAR_END do
        if getFlag(i) == CHARSTATE_FORCE then
            setFlag(i, CHARSTATE_PLAYABLE)
        end
    end
    
    setFlag(c, CHARSTATE_FORCE)
end

function v.warpToMapAs(map, x, y, c, cname) -- TODO make it autodetect cname!
    v.setCharacter(c, true)
    setFlag(FLAG_BLOCK_CHANGE_CHAR, 1) -- so that it does not change even if forced
    setStringFlag(cname .. "_data", "return { x=" .. x .. ", y=" .. y .. ", map=\"" .. map .. "\" }")
    warpAvatar(map, x, y)
end



v._visualsEnt = false -- later either 0 or an entity

function v.avatar_startEmitter(prt)
    if not v._visualsEnt then
        v._visualsEnt = getEntity("logichelp_visuals")
    end
    if v._visualsEnt ~= 0 then
        entity_msg(v._visualsEnt, "start " .. prt)
    end
end

function v.avatar_stopEmitter(prt)
    if v._visualsEnt ~= 0 then
        entity_msg(v._visualsEnt, "stop " .. prt)
    end
end
