
-- -[ linear finite automaton ]-
-- supports an arbitrary amount of states with an adjustable failure tolerance

function v.lfa_create(s, cb, t)
    local self =
    {
        reset = v.lfa_reset,
        input = v.lfa_input,
        trigger = v.lfa_trigger,
        _seq = s,
        _tolerance = tonumber(t) or 0,
        _callback = cb,
        _state = {}, -- last accepted positions, with fails
        _armed = false -- true if at least one accepting state was reached and no further input given
    }
    return self
end

function v.lfa_reset(self)
    self._state = {}
    self._armed = false
end

function v.lfa_trigger(self)
    if self._armed then
        self._callback()
    end
end

local function b2i(b)
    if b then return 1 end
    return 0
end

function v.lfa_input(self, x)
    self._armed = false
    
    for i, state in pairs(self._state) do
        if state then
            local fails = state.fails
            local newpos = state.p + 1
            local want = self._seq[newpos]
            local final = #self._seq == newpos
            local accepted = want == x
            
            --debugLog(string.format("[%d] pos: (%d/%d), want: %d, got: %d, good: %d, final: %d, fails: %d", i, newpos, #self._seq, want, x, b2i(accepted), b2i(final), state.fails))
            
            if accepted then
                state.fails = 0
                if final then
                    self._armed = true
                else
                    state.p = newpos
                end
            else
                state.fails = state.fails + 1
                if state.fails <= self._tolerance then
                    -- still okay
                else
                    self._state[i] = nil -- drop this state
                end
            end
        end
    end
    
    if x == self._seq[1] then
        --debugLog("new lfa state")
        table.insert(self._state, { p = 1, fails = 0 })
    end
end

