-- Copyright (C) 2007, 2010 - Bit-Blot
--
-- This file is part of Aquaria.
--
-- Aquaria is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
--
-- See the GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

if not v then v = {} end
if not AQUARIA_VERSION then dofile("scripts/entities/entityinclude.lua") end

dofile("scripts/inc_util.lua")

local PUSHBACK_DIST = 1000.0
local EXPLODE_DIST = 600.0 -- approx. explosion radius (send "expl" msg to all entities in this range)
local CLAMP_DIST = 6 -- in map squares

local EXPLODE_SOUND = "pistolshrimp-fire"
local EXPLODE_SOUND2 = "mantis-bomb"

v.n = 0

v.timer = 0

v.halftime = false
v.bloating = false

v.pulled = false
v.deltimer = 0
v.fadetimer = 0
v.exploded = false

function init(me)
	setupEntity(me)
	entity_setEntityType(me, ET_ENEMY)
	entity_setTexture(me, "libomb")
	
	entity_setAllDamageTargets(me, false)
	
	entity_setCollideRadius(me, 64)
	
	entity_setState(me, STATE_IDLE)
    
    entity_setUpdateCull(me, -1)
	
	entity_scale(me, 1.5, 1.5)
	
	entity_setCanLeaveWater(me, true)
	
	entity_setProperty(me, EP_MOVABLE, true)
	
	--entity_rotate(me, 360, 1, -1)
    
    loadSound(EXPLODE_SOUND)
    loadSound(EXPLODE_SOUND2)
end

local function tryclamp(me)
    -- try to attach to wall
    if entity_isNearObstruction(me, CLAMP_DIST, OBSCHECK_RANGE) then
        local r = entity_getRotation(me)
        entity_clampToSurface(me)
        entity_rotate(me, r, 0.01) -- HACK: always ensure correct rotation, that may be set wrongly by entity_clampToSurface
        return true
    end
    return false
end

function postInit(me)
	v.n = getNaija()
	entity_setTarget(me, v.n)
    tryclamp(me)
end

local function pushback(me)
	if entity_touchAvatarDamage(me, PUSHBACK_DIST, 0, 1300, 0.2) then
		debugLog("LiBomb - pushback!")
        emote(EMOTE_NAIJAUGH)
	end
end

local function isNearMe(who, me)
    return entity_isEntityInRange(me, who, EXPLODE_DIST)
end

local function collectEnts(e, tab)
    table.insert(tab, e)
end

local function sendExplMsg(who, x)
    debugLog("send expl -> " .. entity_getName(who))
    entity_msg(who, "expl", x)
end

local function crushNearby(me)
    -- seems there is a crashing bug when sending messages to entities just spawned
    -- this may be the case here, as blown up entities may spawn a BrokenPiece upon death.
    -- So, we first collect all nearby entities in a list and process these, and nothing else.
    local ents = {}
    v.forAllEntities(collectEnts, ents, isNearMe, me)
    
    for k,e in pairs(ents) do
        --if e ~= me then
            sendExplMsg(e, 0)
        --end
    end
end

function v.explode(me)
    if v.exploded then
        return
    end
    v.exploded = true
    
    -- prevent crash when the bomb explodes while pulling it
    -- FIXME: add avatar_getPullTarget() or something
    if entity_isBeingPulled(me) then
        avatar_setPullTarget(0)
    end
    
	entity_playSfx(me,  EXPLODE_SOUND, nil, 1.3, nil, nil, 3000)
	entity_playSfx(me, EXPLODE_SOUND2, nil, 1.3, nil, nil, 3000)
    
	pushback(me)
    crushNearby(me)
	shakeCamera(60, 1)
	
	local maxa = 3.14 * 2
	local stp = maxa / 40.0
	local a = 0
	while a < maxa do
		local s = createShot("libomb", me)
		shot_setAimVector(s, math.sin(a), math.cos(a))
		a = a + stp
	end
    
	fade2(1, 0.01, 1, 1, 1)
    entity_alpha(me, 0)
    v.deltimer = 0.2
    v.fadetimer = 0.05
    -- avoid using watch() here, it seems that makes naija invincible
end

function update(me, dt)
	entity_updateMovement(me, dt)
    entity_updateCurrents(me, dt)
	entity_checkSplash(me)
	
	if entity_isUnderWater(me) then
		entity_setWeight(me, 70)
		entity_setMaxSpeed(me, 400)
	else
		entity_setWeight(me, 400)
		entity_setMaxSpeed(me, 800)
	end
	
	v.timer = v.timer + dt
	if v.timer > 5 and not v.halftime then
		v.halftime = true
		--entity_color(me, 1, 1, 1)
		--entity_color(me, 1, 0.0, 0.0, 0.1, -1, 1)
		--entity_offset(me, -10, 0)
		--entity_offset(me, 10, 0, 0.05, -1, 1)
        entity_color(me, 1, 0, 0, 5)
	end
	if v.timer > 8.5 and not v.bloating then
        v.bloating = true
        entity_scale(me, 3, 3, 1.5)
        entity_offset(me, -5, 0)
		entity_offset(me, 5, 0, 0.05, -1, 1)
    end
    if v.timer > 10 then
		v.explode(me)
	end
	
	if entity_isBeingPulled(me) then
		v.pulled = true
	end
	if not entity_isBeingPulled(me) and v.pulled then
		v.pulled = false
        if not tryclamp(me) then
            local vx = entity_velx(me)
            local vy = entity_vely(me)
            
            if entity_isUnderWater(me) then
                entity_clearVel(me)
            end
            entity_addVel(me, vx*0.25, vy*0.25)
            entity_addVel(me, 0, 50)
        end
	end
	
	--[[if not entity_isBeingPulled(me) then
		if entity_touchAvatarDamage(me, entity_getCollideRadius(me, 3), 10, 2000, 0.5) then
			explode(me)
			return
		end
	end]]--
    
    if v.exploded then
        if v.fadetimer > 0 then
            v.fadetimer = v.fadetimer - dt
            if v.fadetimer <= 0 then
                fade2(0, 0.4, 1, 1, 1)
            end
        end
        
        if v.deltimer > 0 then
            v.deltimer = v.deltimer - dt
            if v.deltimer <= 0 then
                emote(EMOTE_NAIJAUGH)
                entity_delete(me)
            end
        end
    end
            
end

function enterState(me)
	if entity_isState(me, STATE_IDLE) then
		entity_animate(me, "idle", -1)
	end
end

function exitState(me)
end

function msg(me, msg)
	if msg == "exp" then
		explode(me)
	end
end

function damage(me, attacker, bone, damageType, dmg)
	return false
end

function animationKey(me, key)
end

function hitSurface(me)
end

function songNote(me, note)
end

function songNoteDone(me, note)
end

function song(me, song)
end

function activate(me)
end

